from typing import Literal, cast

CreateInstanceRequestFeaturesItem = Literal["delete_on_stop"]

CREATE_INSTANCE_REQUEST_FEATURES_ITEM_VALUES: set[CreateInstanceRequestFeaturesItem] = {
    "delete_on_stop",
}


def check_create_instance_request_features_item(value: str) -> CreateInstanceRequestFeaturesItem:
    if value in CREATE_INSTANCE_REQUEST_FEATURES_ITEM_VALUES:
        return cast(CreateInstanceRequestFeaturesItem, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {CREATE_INSTANCE_REQUEST_FEATURES_ITEM_VALUES!r}")



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.delete_autoscale_configuration_policy_response_policies_response import (
        DeleteAutoscaleConfigurationPolicyResponsePoliciesResponse,
    )


T = TypeVar("T", bound="DeleteAutoscaleConfigurationPolicyResponseData")


@_attrs_define
class DeleteAutoscaleConfigurationPolicyResponseData:
    policies: Union[Unset, list["DeleteAutoscaleConfigurationPolicyResponsePoliciesResponse"]] = UNSET
    """ The policies which were deleted by the request. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        policies: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.policies, Unset):
            policies = []
            for policies_item_data in self.policies:
                policies_item = policies_item_data.to_dict()
                policies.append(policies_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if policies is not UNSET:
            field_dict["policies"] = policies

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.delete_autoscale_configuration_policy_response_policies_response import (
            DeleteAutoscaleConfigurationPolicyResponsePoliciesResponse,
        )

        d = dict(src_dict)
        policies = []
        _policies = d.pop("policies", UNSET)
        for policies_item_data in _policies or []:
            policies_item = DeleteAutoscaleConfigurationPolicyResponsePoliciesResponse.from_dict(policies_item_data)

            policies.append(policies_item)

        delete_autoscale_configuration_policy_response_data = cls(
            policies=policies,
        )

        delete_autoscale_configuration_policy_response_data.additional_properties = d
        return delete_autoscale_configuration_policy_response_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.wait_instance_response_waited_instance import WaitInstanceResponseWaitedInstance


T = TypeVar("T", bound="WaitInstanceResponseData")


@_attrs_define
class WaitInstanceResponseData:
    instances: Union[Unset, list["WaitInstanceResponseWaitedInstance"]] = UNSET
    """ The instance(s) which this requested waited on. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        instances: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.instances, Unset):
            instances = []
            for instances_item_data in self.instances:
                instances_item = instances_item_data.to_dict()
                instances.append(instances_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if instances is not UNSET:
            field_dict["instances"] = instances

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.wait_instance_response_waited_instance import WaitInstanceResponseWaitedInstance

        d = dict(src_dict)
        instances = []
        _instances = d.pop("instances", UNSET)
        for instances_item_data in _instances or []:
            instances_item = WaitInstanceResponseWaitedInstance.from_dict(instances_item_data)

            instances.append(instances_item)

        wait_instance_response_data = cls(
            instances=instances,
        )

        wait_instance_response_data.additional_properties = d
        return wait_instance_response_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.certificate_state import CertificateState, check_certificate_state
from ..models.response_status import ResponseStatus, check_response_status
from ..types import UNSET, Unset

T = TypeVar("T", bound="Certificate")


@_attrs_define
class Certificate:
    uuid: Union[Unset, UUID] = UNSET
    """ The UUID of the certificate.

    This is a unique identifier for the certificate that is generated when the
    certificate is created.  The UUID is used to reference the certificate in
    API calls and can be used to identify the certificate in all API calls that
    require an identifier. """
    name: Union[Unset, str] = UNSET
    """ The name of the certificate.

    This is a human-readable name that can be used to identify the certificate.
    The name must be unique within the context of your account.  The name can
    also be used to identify the certificate in API calls. """
    created_at: Union[Unset, datetime.datetime] = UNSET
    """ The time the certificate was created. """
    common_name: Union[Unset, str] = UNSET
    """ The common name (CN) field from the certificate's subject.

    This is typically the primary domain name that the certificate is issued
    for. It represents the main identity that the certificate validates. """
    subject: Union[Unset, str] = UNSET
    """ The complete subject distinguished name (DN) of the certificate.

    This contains the full subject information from the certificate, including
    the common name, organization, organizational unit, locality, state, and
    country. The subject identifies the entity that the certificate is issued to. """
    issuer: Union[Unset, str] = UNSET
    """ The complete issuer distinguished name (DN) of the certificate.

    This identifies the Certificate Authority (CA) that issued the certificate.
    It contains information about the CA including its common name, organization,
    and country. """
    serial_number: Union[Unset, str] = UNSET
    """ The unique serial number assigned to the certificate by the issuing CA.

    This is a unique identifier within the scope of the issuing CA that can be
    used to identify and track the certificate. Serial numbers are typically
    represented as hexadecimal strings. """
    not_before: Union[Unset, datetime.datetime] = UNSET
    """ The date and time when the certificate becomes valid.

    The certificate should not be trusted before this date. This timestamp
    marks the beginning of the certificate's validity period. """
    not_after: Union[Unset, datetime.datetime] = UNSET
    """ The date and time when the certificate expires.

    The certificate should not be trusted after this date. This timestamp
    marks the end of the certificate's validity period. Certificates should
    be renewed before this date to maintain service availability. """
    state: Union[Unset, CertificateState] = UNSET
    """ The current state of the certificate.

    This indicates whether the certificate is pending issuance, valid and
    ready for use, or in an error state. See CertificateState enum for
    detailed state descriptions. """
    status: Union[Unset, ResponseStatus] = UNSET
    """ The response status of an API request. """
    message: Union[Unset, str] = UNSET
    """ An optional message providing additional information about the status.
    This field is only set when this message object is used as a response
    message, and is useful when the status is not `success`. """
    error: Union[Unset, int] = UNSET
    """ An optional error code providing additional information about the status.
    This field is only set when this message object is used as a response
    message, and is useful when the status is not `success`. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        name = self.name

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        common_name = self.common_name

        subject = self.subject

        issuer = self.issuer

        serial_number = self.serial_number

        not_before: Union[Unset, str] = UNSET
        if not isinstance(self.not_before, Unset):
            not_before = self.not_before.isoformat()

        not_after: Union[Unset, str] = UNSET
        if not isinstance(self.not_after, Unset):
            not_after = self.not_after.isoformat()

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status

        message = self.message

        error = self.error

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if name is not UNSET:
            field_dict["name"] = name
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if common_name is not UNSET:
            field_dict["common_name"] = common_name
        if subject is not UNSET:
            field_dict["subject"] = subject
        if issuer is not UNSET:
            field_dict["issuer"] = issuer
        if serial_number is not UNSET:
            field_dict["serial_number"] = serial_number
        if not_before is not UNSET:
            field_dict["not_before"] = not_before
        if not_after is not UNSET:
            field_dict["not_after"] = not_after
        if state is not UNSET:
            field_dict["state"] = state
        if status is not UNSET:
            field_dict["status"] = status
        if message is not UNSET:
            field_dict["message"] = message
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        name = d.pop("name", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        common_name = d.pop("common_name", UNSET)

        subject = d.pop("subject", UNSET)

        issuer = d.pop("issuer", UNSET)

        serial_number = d.pop("serial_number", UNSET)

        _not_before = d.pop("not_before", UNSET)
        not_before: Union[Unset, datetime.datetime]
        if isinstance(_not_before, Unset):
            not_before = UNSET
        else:
            not_before = isoparse(_not_before)

        _not_after = d.pop("not_after", UNSET)
        not_after: Union[Unset, datetime.datetime]
        if isinstance(_not_after, Unset):
            not_after = UNSET
        else:
            not_after = isoparse(_not_after)

        _state = d.pop("state", UNSET)
        state: Union[Unset, CertificateState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = check_certificate_state(_state)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ResponseStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = check_response_status(_status)

        message = d.pop("message", UNSET)

        error = d.pop("error", UNSET)

        certificate = cls(
            uuid=uuid,
            name=name,
            created_at=created_at,
            common_name=common_name,
            subject=subject,
            issuer=issuer,
            serial_number=serial_number,
            not_before=not_before,
            not_after=not_after,
            state=state,
            status=status,
            message=message,
            error=error,
        )

        certificate.additional_properties = d
        return certificate

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ...client import AuthenticatedClient, Client
from ...models.detach_volume_by_uuid_request_body import DetachVolumeByUUIDRequestBody
from ...models.detach_volumes_response import DetachVolumesResponse
from ...types import Response


def _get_kwargs(
    uuid: UUID,
    *,
    body: DetachVolumeByUUIDRequestBody,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": f"/v1/volumes/{uuid}/detach",
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> DetachVolumesResponse:
    response_default = DetachVolumesResponse.from_dict(response.json())

    return response_default


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[DetachVolumesResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: DetachVolumeByUUIDRequestBody,
) -> Response[DetachVolumesResponse]:
    """Detach Volume by UUID

     Detaches a volume by UUID from instances.  If no particular instance is
    specified the volume is detached from all instances.  The instances from
    which to detach must not have the volume mounted.  The API returns an error
    for each instance from which it was unable to detach the volume.  If the
    volume has been created together with an instance, detaching the volume
    will make it persistent (i.e., it survives the deletion of the instance).

    Args:
        uuid (UUID):  Example: c1d2e3f4-5678-90ab-cdef-1234567890ab.
        body (DetachVolumeByUUIDRequestBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DetachVolumesResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: DetachVolumeByUUIDRequestBody,
) -> Optional[DetachVolumesResponse]:
    """Detach Volume by UUID

     Detaches a volume by UUID from instances.  If no particular instance is
    specified the volume is detached from all instances.  The instances from
    which to detach must not have the volume mounted.  The API returns an error
    for each instance from which it was unable to detach the volume.  If the
    volume has been created together with an instance, detaching the volume
    will make it persistent (i.e., it survives the deletion of the instance).

    Args:
        uuid (UUID):  Example: c1d2e3f4-5678-90ab-cdef-1234567890ab.
        body (DetachVolumeByUUIDRequestBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DetachVolumesResponse
    """

    return sync_detailed(
        uuid=uuid,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: DetachVolumeByUUIDRequestBody,
) -> Response[DetachVolumesResponse]:
    """Detach Volume by UUID

     Detaches a volume by UUID from instances.  If no particular instance is
    specified the volume is detached from all instances.  The instances from
    which to detach must not have the volume mounted.  The API returns an error
    for each instance from which it was unable to detach the volume.  If the
    volume has been created together with an instance, detaching the volume
    will make it persistent (i.e., it survives the deletion of the instance).

    Args:
        uuid (UUID):  Example: c1d2e3f4-5678-90ab-cdef-1234567890ab.
        body (DetachVolumeByUUIDRequestBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DetachVolumesResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: DetachVolumeByUUIDRequestBody,
) -> Optional[DetachVolumesResponse]:
    """Detach Volume by UUID

     Detaches a volume by UUID from instances.  If no particular instance is
    specified the volume is detached from all instances.  The instances from
    which to detach must not have the volume mounted.  The API returns an error
    for each instance from which it was unable to detach the volume.  If the
    volume has been created together with an instance, detaching the volume
    will make it persistent (i.e., it survives the deletion of the instance).

    Args:
        uuid (UUID):  Example: c1d2e3f4-5678-90ab-cdef-1234567890ab.
        body (DetachVolumeByUUIDRequestBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DetachVolumesResponse
    """

    return (
        await asyncio_detailed(
            uuid=uuid,
            client=client,
            body=body,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateInstanceRequestVolume")


@_attrs_define
class CreateInstanceRequestVolume:
    """A volume defines a storage volume that can be attached to the instance."""

    at: str
    """ The mount point for the volume in the instance. """
    uuid: Union[Unset, UUID] = UNSET
    """ The UUID of an existing volume.

    If this is the only specified field, then it will look up an existing
    volume by this UUID. """
    name: Union[Unset, str] = UNSET
    """ The name of the volume.

    If this is the only specified field, then it will look up an existing
    volume by this name.  If the volume does not exist, the request will
    fail.  If a new volume is intended to be created, then this field must be
    specified along with the size in MiB and the mount point in the instance. """
    size_mb: Union[Unset, int] = UNSET
    """ The size of the volume when creating a new volume.

    When creating a new volume as part of the instance create request,
    specify the size of the volume in MiB. """
    read_only: Union[Unset, bool] = UNSET
    """ Whether the volume is read-only.

    If this field is set to true, the volume will be mounted as read-only in
    the instance.  This field is optional and defaults to false and is only
    applicable when using an existing volume. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        at = self.at

        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        name = self.name

        size_mb = self.size_mb

        read_only = self.read_only

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "at": at,
            }
        )
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if name is not UNSET:
            field_dict["name"] = name
        if size_mb is not UNSET:
            field_dict["size_mb"] = size_mb
        if read_only is not UNSET:
            field_dict["read_only"] = read_only

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        at = d.pop("at")

        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        name = d.pop("name", UNSET)

        size_mb = d.pop("size_mb", UNSET)

        read_only = d.pop("read_only", UNSET)

        create_instance_request_volume = cls(
            at=at,
            uuid=uuid,
            name=name,
            size_mb=size_mb,
            read_only=read_only,
        )

        create_instance_request_volume.additional_properties = d
        return create_instance_request_volume

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.wait_instance_by_uuid_request_body_state import (
    WaitInstanceByUUIDRequestBodyState,
    check_wait_instance_by_uuid_request_body_state,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="WaitInstanceByUUIDRequestBody")


@_attrs_define
class WaitInstanceByUUIDRequestBody:
    """Wait parameters."""

    state: Union[Unset, WaitInstanceByUUIDRequestBodyState] = UNSET
    """ The desired state to wait for.  Default is `running`. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if state is not UNSET:
            field_dict["state"] = state

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _state = d.pop("state", UNSET)
        state: Union[Unset, WaitInstanceByUUIDRequestBodyState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = check_wait_instance_by_uuid_request_body_state(_state)

        wait_instance_by_uuid_request_body = cls(
            state=state,
        )

        wait_instance_by_uuid_request_body.additional_properties = d
        return wait_instance_by_uuid_request_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...models.delete_instances_response import DeleteInstancesResponse
from ...models.name_or_uuid import NameOrUUID
from ...types import Response


def _get_kwargs(
    *,
    body: list["NameOrUUID"],
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "delete",
        "url": "/v1/instances",
    }

    _kwargs["json"] = []
    for body_item_data in body:
        body_item = body_item_data.to_dict()
        _kwargs["json"].append(body_item)

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> DeleteInstancesResponse:
    response_default = DeleteInstancesResponse.from_dict(response.json())

    return response_default


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[DeleteInstancesResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Response[DeleteInstancesResponse]:
    """Delete Instances

     Delete the specified instance(s) by ID(s) (name or UUID).  After this call
    the name of the instances are no longer valid.  If the instances are
    currently running, they are force-stopped.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteInstancesResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Optional[DeleteInstancesResponse]:
    """Delete Instances

     Delete the specified instance(s) by ID(s) (name or UUID).  After this call
    the name of the instances are no longer valid.  If the instances are
    currently running, they are force-stopped.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteInstancesResponse
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Response[DeleteInstancesResponse]:
    """Delete Instances

     Delete the specified instance(s) by ID(s) (name or UUID).  After this call
    the name of the instances are no longer valid.  If the instances are
    currently running, they are force-stopped.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteInstancesResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Optional[DeleteInstancesResponse]:
    """Delete Instances

     Delete the specified instance(s) by ID(s) (name or UUID).  After this call
    the name of the instances are no longer valid.  If the instances are
    currently running, they are force-stopped.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteInstancesResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...models.delete_service_groups_response import DeleteServiceGroupsResponse
from ...models.name_or_uuid import NameOrUUID
from ...types import Response


def _get_kwargs(
    *,
    body: list["NameOrUUID"],
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "delete",
        "url": "/v1/services",
    }

    _kwargs["json"] = []
    for body_item_data in body:
        body_item = body_item_data.to_dict()
        _kwargs["json"].append(body_item)

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> DeleteServiceGroupsResponse:
    response_default = DeleteServiceGroupsResponse.from_dict(response.json())

    return response_default


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[DeleteServiceGroupsResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Response[DeleteServiceGroupsResponse]:
    """Delete Service Groups

     Delete the specified service group(s).  After this call the name of the
    service group(s) are no longer valid.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteServiceGroupsResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Optional[DeleteServiceGroupsResponse]:
    """Delete Service Groups

     Delete the specified service group(s).  After this call the name of the
    service group(s) are no longer valid.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteServiceGroupsResponse
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Response[DeleteServiceGroupsResponse]:
    """Delete Service Groups

     Delete the specified service group(s).  After this call the name of the
    service group(s) are no longer valid.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteServiceGroupsResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Optional[DeleteServiceGroupsResponse]:
    """Delete Service Groups

     Delete the specified service group(s).  After this call the name of the
    service group(s) are no longer valid.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteServiceGroupsResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from typing import Literal, cast

UpdateInstanceByUUIDRequestBodyOp = Literal["add", "del", "set"]

UPDATE_INSTANCE_BY_UUID_REQUEST_BODY_OP_VALUES: set[UpdateInstanceByUUIDRequestBodyOp] = {
    "add",
    "del",
    "set",
}


def check_update_instance_by_uuid_request_body_op(value: str) -> UpdateInstanceByUUIDRequestBodyOp:
    if value in UPDATE_INSTANCE_BY_UUID_REQUEST_BODY_OP_VALUES:
        return cast(UpdateInstanceByUUIDRequestBodyOp, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {UPDATE_INSTANCE_BY_UUID_REQUEST_BODY_OP_VALUES!r}")



from typing import Literal, cast

UpdateVolumesRequestItemProp = Literal["delete_lock", "quota_policy", "size_mb", "tags"]

UPDATE_VOLUMES_REQUEST_ITEM_PROP_VALUES: set[UpdateVolumesRequestItemProp] = {
    "delete_lock",
    "quota_policy",
    "size_mb",
    "tags",
}


def check_update_volumes_request_item_prop(value: str) -> UpdateVolumesRequestItemProp:
    if value in UPDATE_VOLUMES_REQUEST_ITEM_PROP_VALUES:
        return cast(UpdateVolumesRequestItemProp, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {UPDATE_VOLUMES_REQUEST_ITEM_PROP_VALUES!r}")



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.service_group import ServiceGroup


T = TypeVar("T", bound="CreateServiceGroupResponseData")


@_attrs_define
class CreateServiceGroupResponseData:
    service_groups: Union[Unset, list["ServiceGroup"]] = UNSET
    """ The service group which was created by this request.

    Note: only one service group can be specified in the request, so this
    will always contain a single entry. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        service_groups: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.service_groups, Unset):
            service_groups = []
            for service_groups_item_data in self.service_groups:
                service_groups_item = service_groups_item_data.to_dict()
                service_groups.append(service_groups_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if service_groups is not UNSET:
            field_dict["service_groups"] = service_groups

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.service_group import ServiceGroup

        d = dict(src_dict)
        service_groups = []
        _service_groups = d.pop("service_groups", UNSET)
        for service_groups_item_data in _service_groups or []:
            service_groups_item = ServiceGroup.from_dict(service_groups_item_data)

            service_groups.append(service_groups_item)

        create_service_group_response_data = cls(
            service_groups=service_groups,
        )

        create_service_group_response_data.additional_properties = d
        return create_service_group_response_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="StopInstancesRequestID")


@_attrs_define
class StopInstancesRequestID:
    """An identifier for the instance(s) to start."""

    uuid: UUID
    """ The UUID of the instance to stop.  Mutually exclusive with name. """
    name: str
    """ The name of the instance to stop.  Mutually exclusive with UUID. """
    force: Union[Unset, bool] = UNSET
    """ Whether to immediately force stop the instance. """
    drain_timeout_ms: Union[Unset, int] = UNSET
    """ Timeout for draining connections in milliseconds.  The instance does not
    receive new connections in the draining phase.  The instance is stopped
    when the last connection has been closed or the timeout expired.  The
    maximum timeout may vary.  Use -1 for the largest possible value.

    Note: This endpoint does not block.  Use the wait endpoint for the
    instance to reach the stopped state. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = str(self.uuid)

        name = self.name

        force = self.force

        drain_timeout_ms = self.drain_timeout_ms

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uuid": uuid,
                "name": name,
            }
        )
        if force is not UNSET:
            field_dict["force"] = force
        if drain_timeout_ms is not UNSET:
            field_dict["drain_timeout_ms"] = drain_timeout_ms

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        uuid = UUID(d.pop("uuid"))

        name = d.pop("name")

        force = d.pop("force", UNSET)

        drain_timeout_ms = d.pop("drain_timeout_ms", UNSET)

        stop_instances_request_id = cls(
            uuid=uuid,
            name=name,
            force=force,
            drain_timeout_ms=drain_timeout_ms,
        )

        stop_instances_request_id.additional_properties = d
        return stop_instances_request_id

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...models.create_autoscale_configuration_policy_request import CreateAutoscaleConfigurationPolicyRequest
from ...models.create_autoscale_configuration_policy_response import CreateAutoscaleConfigurationPolicyResponse
from ...types import Response


def _get_kwargs(
    uuid: str,
    *,
    body: CreateAutoscaleConfigurationPolicyRequest,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": f"/v1/services/{uuid}/autoscale/policies",
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> CreateAutoscaleConfigurationPolicyResponse:
    response_default = CreateAutoscaleConfigurationPolicyResponse.from_dict(response.json())

    return response_default


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[CreateAutoscaleConfigurationPolicyResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    uuid: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: CreateAutoscaleConfigurationPolicyRequest,
) -> Response[CreateAutoscaleConfigurationPolicyResponse]:
    """Create Autoscale Configuration Policy

     Add a new autoscale policy to an autoscale configuration given a service
    group UUID.

    Args:
        uuid (str):
        body (CreateAutoscaleConfigurationPolicyRequest): The request message to create an
            autoscale configuration policy for a
            service.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CreateAutoscaleConfigurationPolicyResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    uuid: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: CreateAutoscaleConfigurationPolicyRequest,
) -> Optional[CreateAutoscaleConfigurationPolicyResponse]:
    """Create Autoscale Configuration Policy

     Add a new autoscale policy to an autoscale configuration given a service
    group UUID.

    Args:
        uuid (str):
        body (CreateAutoscaleConfigurationPolicyRequest): The request message to create an
            autoscale configuration policy for a
            service.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CreateAutoscaleConfigurationPolicyResponse
    """

    return sync_detailed(
        uuid=uuid,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    uuid: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: CreateAutoscaleConfigurationPolicyRequest,
) -> Response[CreateAutoscaleConfigurationPolicyResponse]:
    """Create Autoscale Configuration Policy

     Add a new autoscale policy to an autoscale configuration given a service
    group UUID.

    Args:
        uuid (str):
        body (CreateAutoscaleConfigurationPolicyRequest): The request message to create an
            autoscale configuration policy for a
            service.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CreateAutoscaleConfigurationPolicyResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    uuid: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: CreateAutoscaleConfigurationPolicyRequest,
) -> Optional[CreateAutoscaleConfigurationPolicyResponse]:
    """Create Autoscale Configuration Policy

     Add a new autoscale policy to an autoscale configuration given a service
    group UUID.

    Args:
        uuid (str):
        body (CreateAutoscaleConfigurationPolicyRequest): The request message to create an
            autoscale configuration policy for a
            service.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CreateAutoscaleConfigurationPolicyResponse
    """

    return (
        await asyncio_detailed(
            uuid=uuid,
            client=client,
            body=body,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="GetInstanceLogsRequest")


@_attrs_define
class GetInstanceLogsRequest:
    """The request message for getting the logs of an instance by their UUID or
    name.

    """

    uuid: UUID
    """ The UUID of the instance to retrieve logs for.  Mutually exclusive with
    name. """
    name: str
    """ The name of the instance to retrieve logs for.  Mutually exclusive with
    UUID. """
    offset: Union[Unset, int] = UNSET
    """ The byte offset of the log output to receive.  A negative sign makes the
    offset relative to the end of the log. """
    limit: Union[Unset, int] = UNSET
    """ The amount of bytes to return at most. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = str(self.uuid)

        name = self.name

        offset = self.offset

        limit = self.limit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uuid": uuid,
                "name": name,
            }
        )
        if offset is not UNSET:
            field_dict["offset"] = offset
        if limit is not UNSET:
            field_dict["limit"] = limit

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        uuid = UUID(d.pop("uuid"))

        name = d.pop("name")

        offset = d.pop("offset", UNSET)

        limit = d.pop("limit", UNSET)

        get_instance_logs_request = cls(
            uuid=uuid,
            name=name,
            offset=offset,
            limit=limit,
        )

        get_instance_logs_request.additional_properties = d
        return get_instance_logs_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ServiceGroupInstance")


@_attrs_define
class ServiceGroupInstance:
    uuid: Union[Unset, UUID] = UNSET
    """ The UUID of the instance.  This is a unique identifier for the instance
    that is generated when the instance is created.  The UUID is used to
    reference the instance in API calls and can be used to identify the
    instance in all API calls that require an instance identifier. """
    name: Union[Unset, str] = UNSET
    """ The name of the instance.  This is a human-readable name that can be used
    to identify the instance.  The name must be unique within the context of
    your account.  If no name is specified, a random name is generated for
    you.  The name can also be used to identify the instance in API calls. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        name = d.pop("name", UNSET)

        service_group_instance = cls(
            uuid=uuid,
            name=name,
        )

        service_group_instance.additional_properties = d
        return service_group_instance

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...models.name_or_uuid import NameOrUUID
from ...models.wait_instance_response import WaitInstanceResponse
from ...models.wait_instances_state import WaitInstancesState
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    body: list["NameOrUUID"],
    state: Union[Unset, WaitInstancesState] = UNSET,
    timeout_ms: Union[Unset, int] = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    json_state: Union[Unset, str] = UNSET
    if not isinstance(state, Unset):
        json_state = state

    params["state"] = json_state

    params["timeout_ms"] = timeout_ms

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/instances/wait",
        "params": params,
    }

    _kwargs["json"] = []
    for body_item_data in body:
        body_item = body_item_data.to_dict()
        _kwargs["json"].append(body_item)

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> WaitInstanceResponse:
    response_default = WaitInstanceResponse.from_dict(response.json())

    return response_default


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[WaitInstanceResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
    state: Union[Unset, WaitInstancesState] = UNSET,
    timeout_ms: Union[Unset, int] = UNSET,
) -> Response[WaitInstanceResponse]:
    """Wait for Instances States

     Wait for one or more instances to reach certain states by ID(s)
    (name or UUID).

    If the instances are already in the desired states, the request will return
    immediately.  If the instances are not in the desired state, the request will
    block until the instances reach the desired state or the timeout is
    reached.  If the timeout is reached, the request will fail with an error.
    If the timeout is -1, the request will block indefinitely until the
    instances reach the desired states.

    Args:
        state (Union[Unset, WaitInstancesState]):
        timeout_ms (Union[Unset, int]):
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[WaitInstanceResponse]
    """

    kwargs = _get_kwargs(
        body=body,
        state=state,
        timeout_ms=timeout_ms,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
    state: Union[Unset, WaitInstancesState] = UNSET,
    timeout_ms: Union[Unset, int] = UNSET,
) -> Optional[WaitInstanceResponse]:
    """Wait for Instances States

     Wait for one or more instances to reach certain states by ID(s)
    (name or UUID).

    If the instances are already in the desired states, the request will return
    immediately.  If the instances are not in the desired state, the request will
    block until the instances reach the desired state or the timeout is
    reached.  If the timeout is reached, the request will fail with an error.
    If the timeout is -1, the request will block indefinitely until the
    instances reach the desired states.

    Args:
        state (Union[Unset, WaitInstancesState]):
        timeout_ms (Union[Unset, int]):
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        WaitInstanceResponse
    """

    return sync_detailed(
        client=client,
        body=body,
        state=state,
        timeout_ms=timeout_ms,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
    state: Union[Unset, WaitInstancesState] = UNSET,
    timeout_ms: Union[Unset, int] = UNSET,
) -> Response[WaitInstanceResponse]:
    """Wait for Instances States

     Wait for one or more instances to reach certain states by ID(s)
    (name or UUID).

    If the instances are already in the desired states, the request will return
    immediately.  If the instances are not in the desired state, the request will
    block until the instances reach the desired state or the timeout is
    reached.  If the timeout is reached, the request will fail with an error.
    If the timeout is -1, the request will block indefinitely until the
    instances reach the desired states.

    Args:
        state (Union[Unset, WaitInstancesState]):
        timeout_ms (Union[Unset, int]):
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[WaitInstanceResponse]
    """

    kwargs = _get_kwargs(
        body=body,
        state=state,
        timeout_ms=timeout_ms,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
    state: Union[Unset, WaitInstancesState] = UNSET,
    timeout_ms: Union[Unset, int] = UNSET,
) -> Optional[WaitInstanceResponse]:
    """Wait for Instances States

     Wait for one or more instances to reach certain states by ID(s)
    (name or UUID).

    If the instances are already in the desired states, the request will return
    immediately.  If the instances are not in the desired state, the request will
    block until the instances reach the desired state or the timeout is
    reached.  If the timeout is reached, the request will fail with an error.
    If the timeout is -1, the request will block indefinitely until the
    instances reach the desired states.

    Args:
        state (Union[Unset, WaitInstancesState]):
        timeout_ms (Union[Unset, int]):
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        WaitInstanceResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
            state=state,
            timeout_ms=timeout_ms,
        )
    ).parsed



from typing import Literal, cast

UpdateServiceGroupByUUIDRequestBodyProp = Literal["domains", "hard_limit", "services", "soft_limit"]

UPDATE_SERVICE_GROUP_BY_UUID_REQUEST_BODY_PROP_VALUES: set[UpdateServiceGroupByUUIDRequestBodyProp] = {
    "domains",
    "hard_limit",
    "services",
    "soft_limit",
}


def check_update_service_group_by_uuid_request_body_prop(value: str) -> UpdateServiceGroupByUUIDRequestBodyProp:
    if value in UPDATE_SERVICE_GROUP_BY_UUID_REQUEST_BODY_PROP_VALUES:
        return cast(UpdateServiceGroupByUUIDRequestBodyProp, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {UPDATE_SERVICE_GROUP_BY_UUID_REQUEST_BODY_PROP_VALUES!r}"
    )



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ...client import AuthenticatedClient, Client
from ...models.delete_volumes_response import DeleteVolumesResponse
from ...types import Response


def _get_kwargs(
    uuid: UUID,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "delete",
        "url": f"/v1/volumes/{uuid}",
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> DeleteVolumesResponse:
    response_default = DeleteVolumesResponse.from_dict(response.json())

    return response_default


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[DeleteVolumesResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[DeleteVolumesResponse]:
    """Delete Volume by UUID

     Delete the specified volume by its UUID.  If the volume is still attached
    to an instance, the operation fails.  After this call, the IDs associated
    with the volume are no longer valid.

    Args:
        uuid (UUID):  Example: c1d2e3f4-5678-90ab-cdef-1234567890ab.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteVolumesResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[DeleteVolumesResponse]:
    """Delete Volume by UUID

     Delete the specified volume by its UUID.  If the volume is still attached
    to an instance, the operation fails.  After this call, the IDs associated
    with the volume are no longer valid.

    Args:
        uuid (UUID):  Example: c1d2e3f4-5678-90ab-cdef-1234567890ab.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteVolumesResponse
    """

    return sync_detailed(
        uuid=uuid,
        client=client,
    ).parsed


async def asyncio_detailed(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[DeleteVolumesResponse]:
    """Delete Volume by UUID

     Delete the specified volume by its UUID.  If the volume is still attached
    to an instance, the operation fails.  After this call, the IDs associated
    with the volume are no longer valid.

    Args:
        uuid (UUID):  Example: c1d2e3f4-5678-90ab-cdef-1234567890ab.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteVolumesResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[DeleteVolumesResponse]:
    """Delete Volume by UUID

     Delete the specified volume by its UUID.  If the volume is still attached
    to an instance, the operation fails.  After this call, the IDs associated
    with the volume are no longer valid.

    Args:
        uuid (UUID):  Example: c1d2e3f4-5678-90ab-cdef-1234567890ab.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteVolumesResponse
    """

    return (
        await asyncio_detailed(
            uuid=uuid,
            client=client,
        )
    ).parsed



from typing import Literal, cast

AutoscalePolicyAdjustmentType = Literal["change", "exact", "percentage"]

AUTOSCALE_POLICY_ADJUSTMENT_TYPE_VALUES: set[AutoscalePolicyAdjustmentType] = {
    "change",
    "exact",
    "percentage",
}


def check_autoscale_policy_adjustment_type(value: str) -> AutoscalePolicyAdjustmentType:
    if value in AUTOSCALE_POLICY_ADJUSTMENT_TYPE_VALUES:
        return cast(AutoscalePolicyAdjustmentType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {AUTOSCALE_POLICY_ADJUSTMENT_TYPE_VALUES!r}")



from collections.abc import Mapping
from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.update_instances_request_op import UpdateInstancesRequestOp, check_update_instances_request_op
from ..models.update_instances_request_prop import UpdateInstancesRequestProp, check_update_instances_request_prop
from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateInstancesRequest")


@_attrs_define
class UpdateInstancesRequest:
    """The request message for updating one or more instances."""

    prop: UpdateInstancesRequestProp
    """ The property to modify. """
    op: UpdateInstancesRequestOp
    """ The operation to perform on the property. """
    id: Union[Unset, str] = UNSET
    """ (Optional).  A client-provided identifier for tracking this operation in the response. """
    uuid: Union[Unset, UUID] = UNSET
    """ The UUID of the instance to update. Mutually exclusive with name. """
    name: Union[Unset, str] = UNSET
    """ The name of the instance to update. Mutually exclusive with UUID. """
    value: Union[Unset, Any] = UNSET
    """ Represents a dynamically typed value which can be either null, a number, a string, a boolean, a recursive
    struct value, or a list of values. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        prop: str = self.prop

        op: str = self.op

        id = self.id

        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        name = self.name

        value = self.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "prop": prop,
                "op": op,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if name is not UNSET:
            field_dict["name"] = name
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        prop = check_update_instances_request_prop(d.pop("prop"))

        op = check_update_instances_request_op(d.pop("op"))

        id = d.pop("id", UNSET)

        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        name = d.pop("name", UNSET)

        value = d.pop("value", UNSET)

        update_instances_request = cls(
            prop=prop,
            op=op,
            id=id,
            uuid=uuid,
            name=name,
            value=value,
        )

        update_instances_request.additional_properties = d
        return update_instances_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.instance_scale_to_zero_policy import InstanceScaleToZeroPolicy, check_instance_scale_to_zero_policy
from ..types import UNSET, Unset

T = TypeVar("T", bound="InstanceScaleToZero")


@_attrs_define
class InstanceScaleToZero:
    """Scale-to-zero defines the configuration for scaling the instance to zero.
    When an instance is scaled-to-zero it can be either stopped (and fully
    shutdown) or paused wherein the state of the instance is preserved (e.g., RAM
    contents) and the instance can be resumed later without losing its state,
    i.e. "stateful".

    """

    enabled: Union[Unset, bool] = UNSET
    """ Indicates whether scale-to-zero is enabled for the instance. """
    policy: Union[Unset, InstanceScaleToZeroPolicy] = UNSET
    """ The specific policy to use for scaling the instance to zero. """
    stateful: Union[Unset, bool] = UNSET
    """ Whether the instance should be stateful when scaled to zero. If set to
    true, the instance will retain its state (e.g., RAM contents) when scaled
    to zero.  This is useful for instances that need to maintain their state
    across scale-to-zero operations.  If set to false, the instance will lose
    its state when scaled to zero, and it will be restarted from scratch when
    scaled back up. """
    cooldown_time_ms: Union[Unset, int] = UNSET
    """ The cooldown time in milliseconds before the instance can be scaled to
    zero again.  This is useful to prevent rapid scaling to zero and back up,
    which can lead to performance issues or resource exhaustion. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        enabled = self.enabled

        policy: Union[Unset, str] = UNSET
        if not isinstance(self.policy, Unset):
            policy = self.policy

        stateful = self.stateful

        cooldown_time_ms = self.cooldown_time_ms

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if policy is not UNSET:
            field_dict["policy"] = policy
        if stateful is not UNSET:
            field_dict["stateful"] = stateful
        if cooldown_time_ms is not UNSET:
            field_dict["cooldown_time_ms"] = cooldown_time_ms

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        enabled = d.pop("enabled", UNSET)

        _policy = d.pop("policy", UNSET)
        policy: Union[Unset, InstanceScaleToZeroPolicy]
        if isinstance(_policy, Unset):
            policy = UNSET
        else:
            policy = check_instance_scale_to_zero_policy(_policy)

        stateful = d.pop("stateful", UNSET)

        cooldown_time_ms = d.pop("cooldown_time_ms", UNSET)

        instance_scale_to_zero = cls(
            enabled=enabled,
            policy=policy,
            stateful=stateful,
            cooldown_time_ms=cooldown_time_ms,
        )

        instance_scale_to_zero.additional_properties = d
        return instance_scale_to_zero

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .attach_volume_by_uuid_request_body import AttachVolumeByUUIDRequestBody
from .attach_volumes_request import AttachVolumesRequest
from .attach_volumes_request_instance_id import AttachVolumesRequestInstanceID
from .attach_volumes_response import AttachVolumesResponse
from .attach_volumes_response_attached_volume import AttachVolumesResponseAttachedVolume
from .attach_volumes_response_data import AttachVolumesResponseData
from .autoscale_policy import AutoscalePolicy
from .autoscale_policy_adjustment_type import AutoscalePolicyAdjustmentType
from .autoscale_policy_metric import AutoscalePolicyMetric
from .autoscale_policy_step import AutoscalePolicyStep
from .body_instance_id import BodyInstanceID
from .certificate import Certificate
from .certificate_state import CertificateState
from .configuration_instance_create_args import ConfigurationInstanceCreateArgs
from .create_autoscale_configuration_by_service_group_uuid_request import (
    CreateAutoscaleConfigurationByServiceGroupUUIDRequest,
)
from .create_autoscale_configuration_by_service_group_uuid_request_instance_create_args import (
    CreateAutoscaleConfigurationByServiceGroupUUIDRequestInstanceCreateArgs,
)
from .create_autoscale_configuration_policy_request import CreateAutoscaleConfigurationPolicyRequest
from .create_autoscale_configuration_policy_response import CreateAutoscaleConfigurationPolicyResponse
from .create_autoscale_configuration_policy_response_data import CreateAutoscaleConfigurationPolicyResponseData
from .create_autoscale_configuration_policy_response_policy import CreateAutoscaleConfigurationPolicyResponsePolicy
from .create_autoscale_configurations_request_configuration import CreateAutoscaleConfigurationsRequestConfiguration
from .create_autoscale_configurations_response import CreateAutoscaleConfigurationsResponse
from .create_autoscale_configurations_response_configurations_response import (
    CreateAutoscaleConfigurationsResponseConfigurationsResponse,
)
from .create_autoscale_configurations_response_data import CreateAutoscaleConfigurationsResponseData
from .create_certificate_request import CreateCertificateRequest
from .create_certificate_response import CreateCertificateResponse
from .create_certificate_response_data import CreateCertificateResponseData
from .create_instance_request import CreateInstanceRequest
from .create_instance_request_domain import CreateInstanceRequestDomain
from .create_instance_request_env import CreateInstanceRequestEnv
from .create_instance_request_features_item import CreateInstanceRequestFeaturesItem
from .create_instance_request_restart_policy import CreateInstanceRequestRestartPolicy
from .create_instance_request_service_group import CreateInstanceRequestServiceGroup
from .create_instance_request_volume import CreateInstanceRequestVolume
from .create_instance_response import CreateInstanceResponse
from .create_instance_response_data import CreateInstanceResponseData
from .create_service_group_request import CreateServiceGroupRequest
from .create_service_group_request_domain import CreateServiceGroupRequestDomain
from .create_service_group_response import CreateServiceGroupResponse
from .create_service_group_response_data import CreateServiceGroupResponseData
from .create_volume_request import CreateVolumeRequest
from .create_volume_response import CreateVolumeResponse
from .create_volume_response_data import CreateVolumeResponseData
from .create_volume_response_volume import CreateVolumeResponseVolume
from .delete_autoscale_configuration_policy_response import DeleteAutoscaleConfigurationPolicyResponse
from .delete_autoscale_configuration_policy_response_data import DeleteAutoscaleConfigurationPolicyResponseData
from .delete_autoscale_configuration_policy_response_policies_response import (
    DeleteAutoscaleConfigurationPolicyResponsePoliciesResponse,
)
from .delete_autoscale_configurations_response import DeleteAutoscaleConfigurationsResponse
from .delete_autoscale_configurations_response_data import DeleteAutoscaleConfigurationsResponseData
from .delete_autoscale_configurations_response_service_group import DeleteAutoscaleConfigurationsResponseServiceGroup
from .delete_certificates_response import DeleteCertificatesResponse
from .delete_certificates_response_data import DeleteCertificatesResponseData
from .delete_certificates_response_deleted_certificate import DeleteCertificatesResponseDeletedCertificate
from .delete_instances_response import DeleteInstancesResponse
from .delete_instances_response_data import DeleteInstancesResponseData
from .delete_instances_response_deleted_instance import DeleteInstancesResponseDeletedInstance
from .delete_policy_request import DeletePolicyRequest
from .delete_service_groups_response import DeleteServiceGroupsResponse
from .delete_service_groups_response_data import DeleteServiceGroupsResponseData
from .delete_service_groups_response_deleted_service_group import DeleteServiceGroupsResponseDeletedServiceGroup
from .delete_volumes_response import DeleteVolumesResponse
from .delete_volumes_response_data import DeleteVolumesResponseData
from .delete_volumes_response_deleted_volume import DeleteVolumesResponseDeletedVolume
from .detach_volume_by_uuid_request_body import DetachVolumeByUUIDRequestBody
from .detach_volumes_request import DetachVolumesRequest
from .detach_volumes_request_instance_id import DetachVolumesRequestInstanceID
from .detach_volumes_response import DetachVolumesResponse
from .detach_volumes_response_data import DetachVolumesResponseData
from .detach_volumes_response_detached_volume import DetachVolumesResponseDetachedVolume
from .domain import Domain
from .get_autoscale_configuration_policy_request import GetAutoscaleConfigurationPolicyRequest
from .get_autoscale_configuration_policy_response import GetAutoscaleConfigurationPolicyResponse
from .get_autoscale_configuration_policy_response_data import GetAutoscaleConfigurationPolicyResponseData
from .get_autoscale_configuration_policy_response_policy_response import (
    GetAutoscaleConfigurationPolicyResponsePolicyResponse,
)
from .get_autoscale_configurations_response import GetAutoscaleConfigurationsResponse
from .get_autoscale_configurations_response_data import GetAutoscaleConfigurationsResponseData
from .get_autoscale_configurations_response_service_group import GetAutoscaleConfigurationsResponseServiceGroup
from .get_autoscale_configurations_response_status import GetAutoscaleConfigurationsResponseStatus
from .get_certificates_response import GetCertificatesResponse
from .get_certificates_response_data import GetCertificatesResponseData
from .get_image_response import GetImageResponse
from .get_image_response_data import GetImageResponseData
from .get_instance_logs_by_uuid_request_body import GetInstanceLogsByUUIDRequestBody
from .get_instance_logs_request import GetInstanceLogsRequest
from .get_instance_logs_response import GetInstanceLogsResponse
from .get_instance_logs_response_available import GetInstanceLogsResponseAvailable
from .get_instance_logs_response_data import GetInstanceLogsResponseData
from .get_instance_logs_response_logged_instance import GetInstanceLogsResponseLoggedInstance
from .get_instance_logs_response_logged_instance_state import GetInstanceLogsResponseLoggedInstanceState
from .get_instance_logs_response_range import GetInstanceLogsResponseRange
from .get_instance_metrics_response import GetInstanceMetricsResponse
from .get_instance_metrics_response_data import GetInstanceMetricsResponseData
from .get_instance_metrics_response_instance_metrics import GetInstanceMetricsResponseInstanceMetrics
from .get_instances_response import GetInstancesResponse
from .get_instances_response_data import GetInstancesResponseData
from .get_service_groups_response import GetServiceGroupsResponse
from .get_service_groups_response_data import GetServiceGroupsResponseData
from .get_volumes_response import GetVolumesResponse
from .get_volumes_response_data import GetVolumesResponseData
from .healthz_response import HealthzResponse
from .healthz_response_data import HealthzResponseData
from .healthz_response_data_services import HealthzResponseDataServices
from .image import Image
from .image_labels import ImageLabels
from .instance import Instance
from .instance_create_args_instance_create_request_roms import InstanceCreateArgsInstanceCreateRequestRoms
from .instance_env import InstanceEnv
from .instance_instance_service_group import InstanceInstanceServiceGroup
from .instance_instance_volume import InstanceInstanceVolume
from .instance_network_interface import InstanceNetworkInterface
from .instance_restart_policy import InstanceRestartPolicy
from .instance_scale_to_zero import InstanceScaleToZero
from .instance_scale_to_zero_policy import InstanceScaleToZeroPolicy
from .instance_service_group_instance_domain import InstanceServiceGroupInstanceDomain
from .instance_state import InstanceState
from .name_or_uuid import NameOrUUID
from .object_ import Object
from .quotas import Quotas
from .quotas_limits import QuotasLimits
from .quotas_response import QuotasResponse
from .quotas_response_data import QuotasResponseData
from .quotas_stats import QuotasStats
from .response_error import ResponseError
from .response_status import ResponseStatus
from .service import Service
from .service_group import ServiceGroup
from .service_group_instance import ServiceGroupInstance
from .service_group_template import ServiceGroupTemplate
from .service_handlers_item import ServiceHandlersItem
from .start_instance_response import StartInstanceResponse
from .start_instance_response_data import StartInstanceResponseData
from .start_instance_response_started_instance import StartInstanceResponseStartedInstance
from .stop_instance_response import StopInstanceResponse
from .stop_instance_response_data import StopInstanceResponseData
from .stop_instance_response_stopped_instance import StopInstanceResponseStoppedInstance
from .stop_instance_response_stopped_instance_previous_state import StopInstanceResponseStoppedInstancePreviousState
from .stop_instance_response_stopped_instance_state import StopInstanceResponseStoppedInstanceState
from .stop_instances_request_id import StopInstancesRequestID
from .update_instance_by_uuid_request_body import UpdateInstanceByUUIDRequestBody
from .update_instance_by_uuid_request_body_op import UpdateInstanceByUUIDRequestBodyOp
from .update_instance_by_uuid_request_body_prop import UpdateInstanceByUUIDRequestBodyProp
from .update_instances_request import UpdateInstancesRequest
from .update_instances_request_op import UpdateInstancesRequestOp
from .update_instances_request_prop import UpdateInstancesRequestProp
from .update_instances_response import UpdateInstancesResponse
from .update_instances_response_data import UpdateInstancesResponseData
from .update_instances_response_updated_instance import UpdateInstancesResponseUpdatedInstance
from .update_service_group_by_uuid_request_body import UpdateServiceGroupByUUIDRequestBody
from .update_service_group_by_uuid_request_body_op import UpdateServiceGroupByUUIDRequestBodyOp
from .update_service_group_by_uuid_request_body_prop import UpdateServiceGroupByUUIDRequestBodyProp
from .update_service_groups_request_item import UpdateServiceGroupsRequestItem
from .update_service_groups_request_item_op import UpdateServiceGroupsRequestItemOp
from .update_service_groups_request_item_prop import UpdateServiceGroupsRequestItemProp
from .update_service_groups_response import UpdateServiceGroupsResponse
from .update_service_groups_response_data import UpdateServiceGroupsResponseData
from .update_service_groups_response_updated_service_group import UpdateServiceGroupsResponseUpdatedServiceGroup
from .update_volume_by_uuid_request_body import UpdateVolumeByUUIDRequestBody
from .update_volume_by_uuid_request_body_op import UpdateVolumeByUUIDRequestBodyOp
from .update_volume_by_uuid_request_body_prop import UpdateVolumeByUUIDRequestBodyProp
from .update_volumes_request_item import UpdateVolumesRequestItem
from .update_volumes_request_item_op import UpdateVolumesRequestItemOp
from .update_volumes_request_item_prop import UpdateVolumesRequestItemProp
from .update_volumes_response import UpdateVolumesResponse
from .update_volumes_response_data import UpdateVolumesResponseData
from .update_volumes_response_updated_volume import UpdateVolumesResponseUpdatedVolume
from .volume import Volume
from .volume_instance_id import VolumeInstanceID
from .volume_state import VolumeState
from .volume_volume_instance_mount import VolumeVolumeInstanceMount
from .wait_instance_by_uuid_request_body import WaitInstanceByUUIDRequestBody
from .wait_instance_by_uuid_request_body_state import WaitInstanceByUUIDRequestBodyState
from .wait_instance_response import WaitInstanceResponse
from .wait_instance_response_data import WaitInstanceResponseData
from .wait_instance_response_waited_instance import WaitInstanceResponseWaitedInstance
from .wait_instance_response_waited_instance_state import WaitInstanceResponseWaitedInstanceState
from .wait_instances_state import WaitInstancesState

__all__ = (
    "AttachVolumeByUUIDRequestBody",
    "AttachVolumesRequest",
    "AttachVolumesRequestInstanceID",
    "AttachVolumesResponse",
    "AttachVolumesResponseAttachedVolume",
    "AttachVolumesResponseData",
    "AutoscalePolicy",
    "AutoscalePolicyAdjustmentType",
    "AutoscalePolicyMetric",
    "AutoscalePolicyStep",
    "BodyInstanceID",
    "Certificate",
    "CertificateState",
    "ConfigurationInstanceCreateArgs",
    "CreateAutoscaleConfigurationByServiceGroupUUIDRequest",
    "CreateAutoscaleConfigurationByServiceGroupUUIDRequestInstanceCreateArgs",
    "CreateAutoscaleConfigurationPolicyRequest",
    "CreateAutoscaleConfigurationPolicyResponse",
    "CreateAutoscaleConfigurationPolicyResponseData",
    "CreateAutoscaleConfigurationPolicyResponsePolicy",
    "CreateAutoscaleConfigurationsRequestConfiguration",
    "CreateAutoscaleConfigurationsResponse",
    "CreateAutoscaleConfigurationsResponseConfigurationsResponse",
    "CreateAutoscaleConfigurationsResponseData",
    "CreateCertificateRequest",
    "CreateCertificateResponse",
    "CreateCertificateResponseData",
    "CreateInstanceRequest",
    "CreateInstanceRequestDomain",
    "CreateInstanceRequestEnv",
    "CreateInstanceRequestFeaturesItem",
    "CreateInstanceRequestRestartPolicy",
    "CreateInstanceRequestServiceGroup",
    "CreateInstanceRequestVolume",
    "CreateInstanceResponse",
    "CreateInstanceResponseData",
    "CreateServiceGroupRequest",
    "CreateServiceGroupRequestDomain",
    "CreateServiceGroupResponse",
    "CreateServiceGroupResponseData",
    "CreateVolumeRequest",
    "CreateVolumeResponse",
    "CreateVolumeResponseData",
    "CreateVolumeResponseVolume",
    "DeleteAutoscaleConfigurationPolicyResponse",
    "DeleteAutoscaleConfigurationPolicyResponseData",
    "DeleteAutoscaleConfigurationPolicyResponsePoliciesResponse",
    "DeleteAutoscaleConfigurationsResponse",
    "DeleteAutoscaleConfigurationsResponseData",
    "DeleteAutoscaleConfigurationsResponseServiceGroup",
    "DeleteCertificatesResponse",
    "DeleteCertificatesResponseData",
    "DeleteCertificatesResponseDeletedCertificate",
    "DeleteInstancesResponse",
    "DeleteInstancesResponseData",
    "DeleteInstancesResponseDeletedInstance",
    "DeletePolicyRequest",
    "DeleteServiceGroupsResponse",
    "DeleteServiceGroupsResponseData",
    "DeleteServiceGroupsResponseDeletedServiceGroup",
    "DeleteVolumesResponse",
    "DeleteVolumesResponseData",
    "DeleteVolumesResponseDeletedVolume",
    "DetachVolumeByUUIDRequestBody",
    "DetachVolumesRequest",
    "DetachVolumesRequestInstanceID",
    "DetachVolumesResponse",
    "DetachVolumesResponseData",
    "DetachVolumesResponseDetachedVolume",
    "Domain",
    "GetAutoscaleConfigurationPolicyRequest",
    "GetAutoscaleConfigurationPolicyResponse",
    "GetAutoscaleConfigurationPolicyResponseData",
    "GetAutoscaleConfigurationPolicyResponsePolicyResponse",
    "GetAutoscaleConfigurationsResponse",
    "GetAutoscaleConfigurationsResponseData",
    "GetAutoscaleConfigurationsResponseServiceGroup",
    "GetAutoscaleConfigurationsResponseStatus",
    "GetCertificatesResponse",
    "GetCertificatesResponseData",
    "GetImageResponse",
    "GetImageResponseData",
    "GetInstanceLogsByUUIDRequestBody",
    "GetInstanceLogsRequest",
    "GetInstanceLogsResponse",
    "GetInstanceLogsResponseAvailable",
    "GetInstanceLogsResponseData",
    "GetInstanceLogsResponseLoggedInstance",
    "GetInstanceLogsResponseLoggedInstanceState",
    "GetInstanceLogsResponseRange",
    "GetInstanceMetricsResponse",
    "GetInstanceMetricsResponseData",
    "GetInstanceMetricsResponseInstanceMetrics",
    "GetInstancesResponse",
    "GetInstancesResponseData",
    "GetServiceGroupsResponse",
    "GetServiceGroupsResponseData",
    "GetVolumesResponse",
    "GetVolumesResponseData",
    "HealthzResponse",
    "HealthzResponseData",
    "HealthzResponseDataServices",
    "Image",
    "ImageLabels",
    "Instance",
    "InstanceCreateArgsInstanceCreateRequestRoms",
    "InstanceEnv",
    "InstanceInstanceServiceGroup",
    "InstanceInstanceVolume",
    "InstanceNetworkInterface",
    "InstanceRestartPolicy",
    "InstanceScaleToZero",
    "InstanceScaleToZeroPolicy",
    "InstanceServiceGroupInstanceDomain",
    "InstanceState",
    "NameOrUUID",
    "Object",
    "Quotas",
    "QuotasLimits",
    "QuotasResponse",
    "QuotasResponseData",
    "QuotasStats",
    "ResponseError",
    "ResponseStatus",
    "Service",
    "ServiceGroup",
    "ServiceGroupInstance",
    "ServiceGroupTemplate",
    "ServiceHandlersItem",
    "StartInstanceResponse",
    "StartInstanceResponseData",
    "StartInstanceResponseStartedInstance",
    "StopInstanceResponse",
    "StopInstanceResponseData",
    "StopInstanceResponseStoppedInstance",
    "StopInstanceResponseStoppedInstancePreviousState",
    "StopInstanceResponseStoppedInstanceState",
    "StopInstancesRequestID",
    "UpdateInstanceByUUIDRequestBody",
    "UpdateInstanceByUUIDRequestBodyOp",
    "UpdateInstanceByUUIDRequestBodyProp",
    "UpdateInstancesRequest",
    "UpdateInstancesRequestOp",
    "UpdateInstancesRequestProp",
    "UpdateInstancesResponse",
    "UpdateInstancesResponseData",
    "UpdateInstancesResponseUpdatedInstance",
    "UpdateServiceGroupByUUIDRequestBody",
    "UpdateServiceGroupByUUIDRequestBodyOp",
    "UpdateServiceGroupByUUIDRequestBodyProp",
    "UpdateServiceGroupsRequestItem",
    "UpdateServiceGroupsRequestItemOp",
    "UpdateServiceGroupsRequestItemProp",
    "UpdateServiceGroupsResponse",
    "UpdateServiceGroupsResponseData",
    "UpdateServiceGroupsResponseUpdatedServiceGroup",
    "UpdateVolumeByUUIDRequestBody",
    "UpdateVolumeByUUIDRequestBodyOp",
    "UpdateVolumeByUUIDRequestBodyProp",
    "UpdateVolumesRequestItem",
    "UpdateVolumesRequestItemOp",
    "UpdateVolumesRequestItemProp",
    "UpdateVolumesResponse",
    "UpdateVolumesResponseData",
    "UpdateVolumesResponseUpdatedVolume",
    "Volume",
    "VolumeInstanceID",
    "VolumeState",
    "VolumeVolumeInstanceMount",
    "WaitInstanceByUUIDRequestBody",
    "WaitInstanceByUUIDRequestBodyState",
    "WaitInstanceResponse",
    "WaitInstanceResponseData",
    "WaitInstanceResponseWaitedInstance",
    "WaitInstanceResponseWaitedInstanceState",
    "WaitInstancesState",
)



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="CreateInstanceRequestDomain")


@_attrs_define
class CreateInstanceRequestDomain:
    """The domain configuration for the service group.

    A domain defines a publicly accessible domain name for the instance.  If
    the domain name ends with a period `.`, it must be a valid Fully Qualified
    Domain Name (FQDN), otherwise it will become a subdomain of the target
    metro.  The domain can be associated with an existing certificate by
    specifying the certificate's name or UUID.  If no certificate is specified
    and a FQDN is provided, Unikraft Cloud will automatically generate a new
    certificate for the domain based on Let's Encrypt and seek to accomplish a
    DNS-01 challenge.

    """

    name: str
    """ Publicly accessible domain name.

    If this name ends in a period `.` it must be a valid Full Qualified
    Domain Name (FQDN), e.g. `example.com.`; otherwise it will become a
    subdomain of the target metro, e.g. `example` becomes
    `example.fra0.unikraft.app`. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        create_instance_request_domain = cls(
            name=name,
        )

        create_instance_request_domain.additional_properties = d
        return create_instance_request_domain

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.instance_restart_policy import InstanceRestartPolicy, check_instance_restart_policy
from ..models.instance_state import InstanceState, check_instance_state
from ..models.response_status import ResponseStatus, check_response_status
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.instance_env import InstanceEnv
    from ..models.instance_instance_service_group import InstanceInstanceServiceGroup
    from ..models.instance_instance_volume import InstanceInstanceVolume
    from ..models.instance_network_interface import InstanceNetworkInterface
    from ..models.instance_scale_to_zero import InstanceScaleToZero


T = TypeVar("T", bound="Instance")


@_attrs_define
class Instance:
    """An instance is a unikernel virtual machine running an application."""

    uuid: Union[Unset, UUID] = UNSET
    """ The UUID of the instance.

    This is a unique identifier for the instance that is generated when the
    instance is created.  The UUID is used to reference the instance in API
    calls and can be used to identify the instance in all API calls that
    require an instance identifier. """
    name: Union[Unset, str] = UNSET
    """ The name of the instance.

    This is a human-readable name that can be used to identify the instance.
    The name must be unique within the context of your account.  The name can
    also be used to identify the instance in API calls. """
    created_at: Union[Unset, datetime.datetime] = UNSET
    """ The time the instance was created. """
    state: Union[Unset, InstanceState] = UNSET
    """ The state of the instance.  This indicates the current state of the
    instance, such as whether it is running, stopped, or in an error state. """
    private_fqdn: Union[Unset, str] = UNSET
    """ The internal hostname of the instance.  This address can be used privately
    within the Unikraft Cloud network to access the instance.  It is not
    accessible from the public Internet. """
    image: Union[Unset, str] = UNSET
    """ The image used to create the instance.  This is a reference to the
    Unikraft image that was used to create the instance. """
    memory_mb: Union[Unset, int] = UNSET
    """ The amount of memory in megabytes allocated for the instance.  This is the
    total amount of memory that is available to the instance for its
    operations. """
    vcpus: Union[Unset, int] = UNSET
    """ The number of vCPUs allocated for the instance.  This is the total
    number of virtual CPUs that are available to the instance for its
    operations. """
    args: Union[Unset, list[str]] = UNSET
    """ The arguments passed to the instance when it was started.  This is a
    list of command-line arguments that were provided to the instance at
    startup.  These arguments can be used to configure the behavior of the
    instance and its applications. """
    env: Union[Unset, "InstanceEnv"] = UNSET
    """ Environment variables set for the instance. """
    start_count: Union[Unset, int] = UNSET
    """ The total number of times the instance has been started.  This is a counter
    that increments each time the instance is started, regardless of whether it
    was manually stopped or restarted.  This can be useful for tracking the
    usage of the instance over time and/or for debugging purposes. """
    restart_count: Union[Unset, int] = UNSET
    """ The total number of times the instance has been restarted. This is a counter
    that increments each time the instance has been restarted. This can be
    useful for tracking the usage of the instance over time and/or for
    debugging purposes. """
    started_at: Union[Unset, datetime.datetime] = UNSET
    """ The time the instance was started.  This is the timestamp when the
    instance was last started. """
    stopped_at: Union[Unset, datetime.datetime] = UNSET
    """ The time the instance was stopped.  This is the timestamp when the
    instance was last stopped.  If the instance is currently running, this
    field will be empty. """
    uptime_ms: Union[Unset, int] = UNSET
    """ The total amount of time the instance has been running in milliseconds. """
    vmm_start_time_us: Union[Unset, int] = UNSET
    """ (Developer-only).  The time taken between the main controller and the
    beginning of execution of the VMM (Virtual Machine Monitor) measured in
    microseconds.  This field is primarily used for debugging and performance
    analysis purposes. """
    vmm_load_time_us: Union[Unset, int] = UNSET
    """ (Developer-only).  The time it took the VMM (Virtual Machine Monitor) to
    load the instance's kernel and initramfs into VM memory measured in
    microseconds.  This field is primarily used for debugging and performance
    analysis purposes. """
    vmm_ready_time_us: Union[Unset, int] = UNSET
    """ (Developer-only).  The time taken for the VMM (Virtual Machine Monitor) to
    become ready to execute the instance measured in microseconds.  This is the
    time from when the VMM started until it was ready to execute the instance's
    code.  This field is primarily used for debugging and performance analysis
    purposes. """
    boot_time_us: Union[Unset, int] = UNSET
    """ The boot time of the instance in microseconds.  We take a pragmatic
    approach is to define the boot time.  We calculate this as the difference
    in time between the moment the virtualization toolstack is invoked to
    respond to a VM boot request and the moment the OS starts executing user
    code (i.e., the end of the guest OS boot process).  This is essentially the
    time that a user would experience in a deployment, minus the application
    initialization time, which we leave out since it is independent from the
    OS. """
    net_time_us: Union[Unset, int] = UNSET
    """ This is the time it took for the user-level application to start listening
    on a non-localhost port measured in microseconds.  This is the time from
    when the instance started until it reasonably ready to start responding to
    network requests.  This is useful for measuring the time it takes for the
    instance to become operationally ready. """
    stop_reason: Union[Unset, int] = UNSET
    """ The instance stop reason.

    Provides reason as to why an instance is stopped or in the process of
    shutting down.  The stop reason is a bitmask that tells you the origin of
    the shutdown:

    | Bit     | 4          | 3          | 2          | 1          | 0 (LSB)      |
    |---------|------------|------------|------------|------------|--------------|
    | Purpose | [F]orced   | [U]ser     | [P]latform | [A]pp      | [K]ernel     |

    - **Forced**:   This was a force stop.  A forced stop does not give the
                    instance a chance to perform a clean shutdown.  Bits 0
                    (Kernel) and 1 (App) can thus never be set for forced
                    shutdowns.  Consequently, there won't be an `exit_code` or
                    `stop_code`.
    - **User**:     Stop initiated by user, e.g. via an API call.
    - **Platform**: Stop initiated by platform, e.g. an autoscale policy.
    - **App**:      The Application exited.  The `exit_code` field will be set.
    - **Kernel**:   The kernel exited.  The `stop_code` field will be set.

    For example, the stop reason will contain the following values in the given
    scenarios:

    | Value | Bitmask | Aliases | Scenario |
    |-------|---------|---------|----------|
    | 28    | `11100` | `FUP--` | Forced user-initiated shutdown. |
    | 15    | `01111` | `-UPAK` | Regular user-initiated shutdown. The application and kernel have exited. The
    exit_code and stop_code indicate if the application and kernel shut down cleanly. |
    | 13    | `01101` | `-UP-K` | The user initiated a shutdown but the application was forcefully killed by the
    kernel during shutdown. This can be the case if the image does not support a clean application exit or the
    application crashed after receiving a termination signal. The exit_code won’t be present in this scenario. |
    | 7     | `00111` | `--PAK` | Unikraft Cloud initiated the shutdown, for example, due to scale-to-zero. The
    application and kernel have exited. The exit_code and stop_code indicate if the application and kernel shut down
    cleanly. |
    | 3     | `00011` | `---AK` | The application exited. The exit_code and stop_code indicate if the application
    and kernel shut down cleanly. |
    | 1     | `00001` | `----K` | The instance likely expierenced a fatal crash and the stop_code contains more
    information about the cause of the crash. |
    | 0     | `00000` | `-----` | The stop reason is unknown. | """
    exit_code: Union[Unset, int] = UNSET
    """ The application exit code.

    This is the code which the application returns upon leaving its main entry
    point.  The encoding of the exit code is application specific.  See the
    documentation of the application for more details.  Usually, an exit code
    of `0` indicates success / no failure. """
    stop_code: Union[Unset, int] = UNSET
    """ The kernel stop code.

    This value encodes multiple details about the stop irrespective of the
    application.

    ```
    MSB                                                     LSB
    ┌──────────────┬──────────┬──────────┬───────────┬────────┐
    │ 31 ────── 24 │ 23 ── 16 │    15    │ 14 ──── 8 │ 7 ── 0 │
    ├──────────────┼──────────┼──────────┼───────────┼────────┤
    │ reserved[^1] │ errno    │ shutdown │ initlevel │ reason │
    └──────────────┴──────────┴──────────┴───────────┴────────┘
    ```

    - **errno**:     The application errno, using Linux's errno.h values.
                     (Optional, can be 0.)
    - **shutdown**:  Whether the shutdown originated from the inittable (0) or
                     from the termtable (1).
    - **initlevel**: The initlevel at the time of the stop.
    - **reason**:    The reason for the stop.  See `StopCodeReason`.

    [^1]: Reserved for future use. """
    restart_policy: Union[Unset, InstanceRestartPolicy] = UNSET
    """ The restart configuration for the instance.

    When an instance stops either because the application exits or the instance
    crashes, Unikraft Cloud can auto-restart your instance.  Auto-restarts are
    performed according to the restart policy configured for a particular
    instance.

    The policy can have the following values:

    | Policy       | Description |
    |--------------|-------------|
    | `never`      | Never restart the instance (default). |
    | `always`     | Always restart the instance when the stop is initiated from within the instance (i.e., the
    application exits or the instance crashes). |
    | `on-failure` | Only restart the instance if it crashes. |

    When an instance stops, the stop reason and the configured restart policy
    are evaluated to decide if a restart should be performed.  Unikraft Cloud
    uses an exponential back-off delay (immediate, 5s, 10s, 20s, 40s, ..., 5m)
    to slow down restarts in tight crash loops.  If an instance runs without
    problems for 10s the back-off delay is reset and the restart sequence ends.

    The `restart.attempt` attribute reported in counts the number of restarts
    performed in the current sequence.  The `restart.next_at` field indicates
    when the next restart will take place if a back-off delay is in effect.

    A manual start or stop of the instance aborts the restart sequence and
    resets the back-off delay. """
    scale_to_zero: Union[Unset, "InstanceScaleToZero"] = UNSET
    """ Scale-to-zero defines the configuration for scaling the instance to zero.
    When an instance is scaled-to-zero it can be either stopped (and fully
    shutdown) or paused wherein the state of the instance is preserved (e.g., RAM
    contents) and the instance can be resumed later without losing its state,
    i.e. "stateful". """
    volumes: Union[Unset, list["InstanceInstanceVolume"]] = UNSET
    """ The list of volumes attached to the instance. """
    service_group: Union[Unset, "InstanceInstanceServiceGroup"] = UNSET
    """ The service group configuration for the instance.

    This is a reference to the service group that the instance is part of.  The
    service group defines the services (e.g. ports, connection handling) that
    the instance exposes and how they are configured. """
    network_interfaces: Union[Unset, list["InstanceNetworkInterface"]] = UNSET
    """ The network interfaces of the instance. """
    tags: Union[Unset, list[str]] = UNSET
    """ The tags associated with the instance. """
    status: Union[Unset, ResponseStatus] = UNSET
    """ The response status of an API request. """
    message: Union[Unset, str] = UNSET
    """ An optional message providing additional information about the status.
    This field is only set when this message object is used as a response
    message, and is useful when the status is not `success`. """
    error: Union[Unset, int] = UNSET
    """ An optional error code providing additional information about the status.
    This field is only set when this message object is used as a response
    message, and is useful when the status is not `success`. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        name = self.name

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state

        private_fqdn = self.private_fqdn

        image = self.image

        memory_mb = self.memory_mb

        vcpus = self.vcpus

        args: Union[Unset, list[str]] = UNSET
        if not isinstance(self.args, Unset):
            args = self.args

        env: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.env, Unset):
            env = self.env.to_dict()

        start_count = self.start_count

        restart_count = self.restart_count

        started_at: Union[Unset, str] = UNSET
        if not isinstance(self.started_at, Unset):
            started_at = self.started_at.isoformat()

        stopped_at: Union[Unset, str] = UNSET
        if not isinstance(self.stopped_at, Unset):
            stopped_at = self.stopped_at.isoformat()

        uptime_ms = self.uptime_ms

        vmm_start_time_us = self.vmm_start_time_us

        vmm_load_time_us = self.vmm_load_time_us

        vmm_ready_time_us = self.vmm_ready_time_us

        boot_time_us = self.boot_time_us

        net_time_us = self.net_time_us

        stop_reason = self.stop_reason

        exit_code = self.exit_code

        stop_code = self.stop_code

        restart_policy: Union[Unset, str] = UNSET
        if not isinstance(self.restart_policy, Unset):
            restart_policy = self.restart_policy

        scale_to_zero: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.scale_to_zero, Unset):
            scale_to_zero = self.scale_to_zero.to_dict()

        volumes: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.volumes, Unset):
            volumes = []
            for volumes_item_data in self.volumes:
                volumes_item = volumes_item_data.to_dict()
                volumes.append(volumes_item)

        service_group: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.service_group, Unset):
            service_group = self.service_group.to_dict()

        network_interfaces: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.network_interfaces, Unset):
            network_interfaces = []
            for network_interfaces_item_data in self.network_interfaces:
                network_interfaces_item = network_interfaces_item_data.to_dict()
                network_interfaces.append(network_interfaces_item)

        tags: Union[Unset, list[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status

        message = self.message

        error = self.error

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if name is not UNSET:
            field_dict["name"] = name
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if state is not UNSET:
            field_dict["state"] = state
        if private_fqdn is not UNSET:
            field_dict["private_fqdn"] = private_fqdn
        if image is not UNSET:
            field_dict["image"] = image
        if memory_mb is not UNSET:
            field_dict["memory_mb"] = memory_mb
        if vcpus is not UNSET:
            field_dict["vcpus"] = vcpus
        if args is not UNSET:
            field_dict["args"] = args
        if env is not UNSET:
            field_dict["env"] = env
        if start_count is not UNSET:
            field_dict["start_count"] = start_count
        if restart_count is not UNSET:
            field_dict["restart_count"] = restart_count
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if stopped_at is not UNSET:
            field_dict["stopped_at"] = stopped_at
        if uptime_ms is not UNSET:
            field_dict["uptime_ms"] = uptime_ms
        if vmm_start_time_us is not UNSET:
            field_dict["vmm_start_time_us"] = vmm_start_time_us
        if vmm_load_time_us is not UNSET:
            field_dict["vmm_load_time_us"] = vmm_load_time_us
        if vmm_ready_time_us is not UNSET:
            field_dict["vmm_ready_time_us"] = vmm_ready_time_us
        if boot_time_us is not UNSET:
            field_dict["boot_time_us"] = boot_time_us
        if net_time_us is not UNSET:
            field_dict["net_time_us"] = net_time_us
        if stop_reason is not UNSET:
            field_dict["stop_reason"] = stop_reason
        if exit_code is not UNSET:
            field_dict["exit_code"] = exit_code
        if stop_code is not UNSET:
            field_dict["stop_code"] = stop_code
        if restart_policy is not UNSET:
            field_dict["restart_policy"] = restart_policy
        if scale_to_zero is not UNSET:
            field_dict["scale_to_zero"] = scale_to_zero
        if volumes is not UNSET:
            field_dict["volumes"] = volumes
        if service_group is not UNSET:
            field_dict["service_group"] = service_group
        if network_interfaces is not UNSET:
            field_dict["network_interfaces"] = network_interfaces
        if tags is not UNSET:
            field_dict["tags"] = tags
        if status is not UNSET:
            field_dict["status"] = status
        if message is not UNSET:
            field_dict["message"] = message
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.instance_env import InstanceEnv
        from ..models.instance_instance_service_group import InstanceInstanceServiceGroup
        from ..models.instance_instance_volume import InstanceInstanceVolume
        from ..models.instance_network_interface import InstanceNetworkInterface
        from ..models.instance_scale_to_zero import InstanceScaleToZero

        d = dict(src_dict)
        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        name = d.pop("name", UNSET)

        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _state = d.pop("state", UNSET)
        state: Union[Unset, InstanceState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = check_instance_state(_state)

        private_fqdn = d.pop("private_fqdn", UNSET)

        image = d.pop("image", UNSET)

        memory_mb = d.pop("memory_mb", UNSET)

        vcpus = d.pop("vcpus", UNSET)

        args = cast(list[str], d.pop("args", UNSET))

        _env = d.pop("env", UNSET)
        env: Union[Unset, InstanceEnv]
        if isinstance(_env, Unset):
            env = UNSET
        else:
            env = InstanceEnv.from_dict(_env)

        start_count = d.pop("start_count", UNSET)

        restart_count = d.pop("restart_count", UNSET)

        _started_at = d.pop("started_at", UNSET)
        started_at: Union[Unset, datetime.datetime]
        if isinstance(_started_at, Unset):
            started_at = UNSET
        else:
            started_at = isoparse(_started_at)

        _stopped_at = d.pop("stopped_at", UNSET)
        stopped_at: Union[Unset, datetime.datetime]
        if isinstance(_stopped_at, Unset):
            stopped_at = UNSET
        else:
            stopped_at = isoparse(_stopped_at)

        uptime_ms = d.pop("uptime_ms", UNSET)

        vmm_start_time_us = d.pop("vmm_start_time_us", UNSET)

        vmm_load_time_us = d.pop("vmm_load_time_us", UNSET)

        vmm_ready_time_us = d.pop("vmm_ready_time_us", UNSET)

        boot_time_us = d.pop("boot_time_us", UNSET)

        net_time_us = d.pop("net_time_us", UNSET)

        stop_reason = d.pop("stop_reason", UNSET)

        exit_code = d.pop("exit_code", UNSET)

        stop_code = d.pop("stop_code", UNSET)

        _restart_policy = d.pop("restart_policy", UNSET)
        restart_policy: Union[Unset, InstanceRestartPolicy]
        if isinstance(_restart_policy, Unset):
            restart_policy = UNSET
        else:
            restart_policy = check_instance_restart_policy(_restart_policy)

        _scale_to_zero = d.pop("scale_to_zero", UNSET)
        scale_to_zero: Union[Unset, InstanceScaleToZero]
        if isinstance(_scale_to_zero, Unset):
            scale_to_zero = UNSET
        else:
            scale_to_zero = InstanceScaleToZero.from_dict(_scale_to_zero)

        volumes = []
        _volumes = d.pop("volumes", UNSET)
        for volumes_item_data in _volumes or []:
            volumes_item = InstanceInstanceVolume.from_dict(volumes_item_data)

            volumes.append(volumes_item)

        _service_group = d.pop("service_group", UNSET)
        service_group: Union[Unset, InstanceInstanceServiceGroup]
        if isinstance(_service_group, Unset):
            service_group = UNSET
        else:
            service_group = InstanceInstanceServiceGroup.from_dict(_service_group)

        network_interfaces = []
        _network_interfaces = d.pop("network_interfaces", UNSET)
        for network_interfaces_item_data in _network_interfaces or []:
            network_interfaces_item = InstanceNetworkInterface.from_dict(network_interfaces_item_data)

            network_interfaces.append(network_interfaces_item)

        tags = cast(list[str], d.pop("tags", UNSET))

        _status = d.pop("status", UNSET)
        status: Union[Unset, ResponseStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = check_response_status(_status)

        message = d.pop("message", UNSET)

        error = d.pop("error", UNSET)

        instance = cls(
            uuid=uuid,
            name=name,
            created_at=created_at,
            state=state,
            private_fqdn=private_fqdn,
            image=image,
            memory_mb=memory_mb,
            vcpus=vcpus,
            args=args,
            env=env,
            start_count=start_count,
            restart_count=restart_count,
            started_at=started_at,
            stopped_at=stopped_at,
            uptime_ms=uptime_ms,
            vmm_start_time_us=vmm_start_time_us,
            vmm_load_time_us=vmm_load_time_us,
            vmm_ready_time_us=vmm_ready_time_us,
            boot_time_us=boot_time_us,
            net_time_us=net_time_us,
            stop_reason=stop_reason,
            exit_code=exit_code,
            stop_code=stop_code,
            restart_policy=restart_policy,
            scale_to_zero=scale_to_zero,
            volumes=volumes,
            service_group=service_group,
            network_interfaces=network_interfaces,
            tags=tags,
            status=status,
            message=message,
            error=error,
        )

        instance.additional_properties = d
        return instance

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...models.delete_volumes_response import DeleteVolumesResponse
from ...models.name_or_uuid import NameOrUUID
from ...types import Response


def _get_kwargs(
    *,
    body: list["NameOrUUID"],
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "delete",
        "url": "/v1/volumes",
    }

    _kwargs["json"] = []
    for body_item_data in body:
        body_item = body_item_data.to_dict()
        _kwargs["json"].append(body_item)

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> DeleteVolumesResponse:
    response_default = DeleteVolumesResponse.from_dict(response.json())

    return response_default


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[DeleteVolumesResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Response[DeleteVolumesResponse]:
    """Delete Volumes

     Delete one or more volumes by their UUID(s) or name(s).  If the volumes are
    still attached to an instance, the operation fails.  After this call, the
    IDs associated with the volumes are no longer valid.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteVolumesResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Optional[DeleteVolumesResponse]:
    """Delete Volumes

     Delete one or more volumes by their UUID(s) or name(s).  If the volumes are
    still attached to an instance, the operation fails.  After this call, the
    IDs associated with the volumes are no longer valid.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteVolumesResponse
    """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Response[DeleteVolumesResponse]:
    """Delete Volumes

     Delete one or more volumes by their UUID(s) or name(s).  If the volumes are
    still attached to an instance, the operation fails.  After this call, the
    IDs associated with the volumes are no longer valid.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteVolumesResponse]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: list["NameOrUUID"],
) -> Optional[DeleteVolumesResponse]:
    """Delete Volumes

     Delete one or more volumes by their UUID(s) or name(s).  If the volumes are
    still attached to an instance, the operation fails.  After this call, the
    IDs associated with the volumes are no longer valid.

    Args:
        body (list['NameOrUUID']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteVolumesResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.autoscale_policy import AutoscalePolicy


T = TypeVar("T", bound="CreateAutoscaleConfigurationPolicyRequest")


@_attrs_define
class CreateAutoscaleConfigurationPolicyRequest:
    """The request message to create an autoscale configuration policy for a
    service.

    """

    name: str
    """ The Name of the service to add a policy to. """
    type_: "AutoscalePolicy"
    """ AutoscalePolicy defines the autoscale policy for a service.
    Right now it contains fields from both the `ondemand` and `step` policies.
    They are marked both as optional, so only one of them should be set at a
    time. This is a current limitation of the API design. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        type_ = self.type_.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "type": type_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.autoscale_policy import AutoscalePolicy

        d = dict(src_dict)
        name = d.pop("name")

        type_ = AutoscalePolicy.from_dict(d.pop("type"))

        create_autoscale_configuration_policy_request = cls(
            name=name,
            type_=type_,
        )

        create_autoscale_configuration_policy_request.additional_properties = d
        return create_autoscale_configuration_policy_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Domain")


@_attrs_define
class Domain:
    """A domain name.

    Domain names are completely specified with all labels in the hierarchy of the
    DNS, having no parts omitted.  The domain can be associated with an existing
    certificate by specifying the certificate's name or UUID.  If no certificate
    is specified and a FQDN is provided, Unikraft Cloud will automatically
    generate a new certificate for the domain based on Let's Encrypt and seek to
    accomplish a DNS-01 challenge.

    """

    fqdn: Union[Unset, str] = UNSET
    """ Publicly accessible domain name.  If this name ends in a period `.` it must
    be a valid Full Qualified Domain Name (FQDN), otherwise it will become a
    subdomain of the target metro. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        fqdn = self.fqdn

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if fqdn is not UNSET:
            field_dict["fqdn"] = fqdn

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        fqdn = d.pop("fqdn", UNSET)

        domain = cls(
            fqdn=fqdn,
        )

        domain.additional_properties = d
        return domain

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, cast

InstanceScaleToZeroPolicy = Literal["idle", "off", "on"]

INSTANCE_SCALE_TO_ZERO_POLICY_VALUES: set[InstanceScaleToZeroPolicy] = {
    "idle",
    "off",
    "on",
}


def check_instance_scale_to_zero_policy(value: str) -> InstanceScaleToZeroPolicy:
    if value in INSTANCE_SCALE_TO_ZERO_POLICY_VALUES:
        return cast(InstanceScaleToZeroPolicy, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {INSTANCE_SCALE_TO_ZERO_POLICY_VALUES!r}")



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.response_status import ResponseStatus, check_response_status
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.delete_certificates_response_data import DeleteCertificatesResponseData
    from ..models.response_error import ResponseError


T = TypeVar("T", bound="DeleteCertificatesResponse")


@_attrs_define
class DeleteCertificatesResponse:
    """The response message for deleting of one or more certificate(s) given their
    UUID(s) or name(s).

    """

    status: Union[Unset, ResponseStatus] = UNSET
    """ The response status of an API request. """
    message: Union[Unset, str] = UNSET
    """ An optional message providing additional information about the status.
    This field is useful when the status is not `success`. """
    data: Union[Unset, "DeleteCertificatesResponseData"] = UNSET
    errors: Union[Unset, list["ResponseError"]] = UNSET
    """ A list of errors which may have occurred during the request. """
    op_time_us: Union[Unset, int] = UNSET
    """ The operation time in microseconds.  This is the time it took to process
    the request and generate the response. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status

        message = self.message

        data: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.data, Unset):
            data = self.data.to_dict()

        errors: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.errors, Unset):
            errors = []
            for errors_item_data in self.errors:
                errors_item = errors_item_data.to_dict()
                errors.append(errors_item)

        op_time_us = self.op_time_us

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status
        if message is not UNSET:
            field_dict["message"] = message
        if data is not UNSET:
            field_dict["data"] = data
        if errors is not UNSET:
            field_dict["errors"] = errors
        if op_time_us is not UNSET:
            field_dict["op_time_us"] = op_time_us

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.delete_certificates_response_data import DeleteCertificatesResponseData
        from ..models.response_error import ResponseError

        d = dict(src_dict)
        _status = d.pop("status", UNSET)
        status: Union[Unset, ResponseStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = check_response_status(_status)

        message = d.pop("message", UNSET)

        _data = d.pop("data", UNSET)
        data: Union[Unset, DeleteCertificatesResponseData]
        if isinstance(_data, Unset):
            data = UNSET
        else:
            data = DeleteCertificatesResponseData.from_dict(_data)

        errors = []
        _errors = d.pop("errors", UNSET)
        for errors_item_data in _errors or []:
            errors_item = ResponseError.from_dict(errors_item_data)

            errors.append(errors_item)

        op_time_us = d.pop("op_time_us", UNSET)

        delete_certificates_response = cls(
            status=status,
            message=message,
            data=data,
            errors=errors,
            op_time_us=op_time_us,
        )

        delete_certificates_response.additional_properties = d
        return delete_certificates_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, cast

InstanceState = Literal["draining", "running", "standby", "starting", "stopped", "stopping"]

INSTANCE_STATE_VALUES: set[InstanceState] = {
    "draining",
    "running",
    "standby",
    "starting",
    "stopped",
    "stopping",
}


def check_instance_state(value: str) -> InstanceState:
    if value in INSTANCE_STATE_VALUES:
        return cast(InstanceState, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {INSTANCE_STATE_VALUES!r}")



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="InstanceCreateArgsInstanceCreateRequestRoms")


@_attrs_define
class InstanceCreateArgsInstanceCreateRequestRoms:
    image: str
    """ The image of the ROM to use for the autoscale configuration. """
    name: Union[Unset, str] = UNSET
    """ The name of the ROM to use for the autoscale configuration. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        image = self.image

        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "image": image,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        image = d.pop("image")

        name = d.pop("name", UNSET)

        instance_create_args_instance_create_request_roms = cls(
            image=image,
            name=name,
        )

        instance_create_args_instance_create_request_roms.additional_properties = d
        return instance_create_args_instance_create_request_roms

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ...client import AuthenticatedClient, Client
from ...models.delete_autoscale_configurations_response import DeleteAutoscaleConfigurationsResponse
from ...types import Response


def _get_kwargs(
    uuid: UUID,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "delete",
        "url": f"/v1/services/{uuid}/autoscale",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> DeleteAutoscaleConfigurationsResponse:
    response_default = DeleteAutoscaleConfigurationsResponse.from_dict(response.json())

    return response_default


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[DeleteAutoscaleConfigurationsResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[DeleteAutoscaleConfigurationsResponse]:
    """Delete Autoscale Configurations by Service Group UUID

     Delete the autoscale configuration for the service group given its UUID.

    Unikraft Cloud will immediately drain all connections from all instances
    that have been created by autoscale and delete the instances afterwards.
    The draining phase is allowed to take at most `cooldown_time_ms`
    milliseconds after which remaining connections are forcefully closed.  The
    master instance is never deleted.  However, deleting the autoscale
    configuration causes the master instance to start if it is stopped.

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteAutoscaleConfigurationsResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[DeleteAutoscaleConfigurationsResponse]:
    """Delete Autoscale Configurations by Service Group UUID

     Delete the autoscale configuration for the service group given its UUID.

    Unikraft Cloud will immediately drain all connections from all instances
    that have been created by autoscale and delete the instances afterwards.
    The draining phase is allowed to take at most `cooldown_time_ms`
    milliseconds after which remaining connections are forcefully closed.  The
    master instance is never deleted.  However, deleting the autoscale
    configuration causes the master instance to start if it is stopped.

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteAutoscaleConfigurationsResponse
    """

    return sync_detailed(
        uuid=uuid,
        client=client,
    ).parsed


async def asyncio_detailed(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[DeleteAutoscaleConfigurationsResponse]:
    """Delete Autoscale Configurations by Service Group UUID

     Delete the autoscale configuration for the service group given its UUID.

    Unikraft Cloud will immediately drain all connections from all instances
    that have been created by autoscale and delete the instances afterwards.
    The draining phase is allowed to take at most `cooldown_time_ms`
    milliseconds after which remaining connections are forcefully closed.  The
    master instance is never deleted.  However, deleting the autoscale
    configuration causes the master instance to start if it is stopped.

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteAutoscaleConfigurationsResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Optional[DeleteAutoscaleConfigurationsResponse]:
    """Delete Autoscale Configurations by Service Group UUID

     Delete the autoscale configuration for the service group given its UUID.

    Unikraft Cloud will immediately drain all connections from all instances
    that have been created by autoscale and delete the instances afterwards.
    The draining phase is allowed to take at most `cooldown_time_ms`
    milliseconds after which remaining connections are forcefully closed.  The
    master instance is never deleted.  However, deleting the autoscale
    configuration causes the master instance to start if it is stopped.

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteAutoscaleConfigurationsResponse
    """

    return (
        await asyncio_detailed(
            uuid=uuid,
            client=client,
        )
    ).parsed



from typing import Literal, cast

StopInstanceResponseStoppedInstanceState = Literal["draining", "running", "standby", "starting", "stopped", "stopping"]

STOP_INSTANCE_RESPONSE_STOPPED_INSTANCE_STATE_VALUES: set[StopInstanceResponseStoppedInstanceState] = {
    "draining",
    "running",
    "standby",
    "starting",
    "stopped",
    "stopping",
}


def check_stop_instance_response_stopped_instance_state(value: str) -> StopInstanceResponseStoppedInstanceState:
    if value in STOP_INSTANCE_RESPONSE_STOPPED_INSTANCE_STATE_VALUES:
        return cast(StopInstanceResponseStoppedInstanceState, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_INSTANCE_RESPONSE_STOPPED_INSTANCE_STATE_VALUES!r}"
    )



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.quotas import Quotas


T = TypeVar("T", bound="QuotasResponseData")


@_attrs_define
class QuotasResponseData:
    quotas: Union[Unset, list["Quotas"]] = UNSET
    """ The quota(s) which were retrieved by the request. """
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        quotas: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.quotas, Unset):
            quotas = []
            for quotas_item_data in self.quotas:
                quotas_item = quotas_item_data.to_dict()
                quotas.append(quotas_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if quotas is not UNSET:
            field_dict["quotas"] = quotas

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.quotas import Quotas

        d = dict(src_dict)
        quotas = []
        _quotas = d.pop("quotas", UNSET)
        for quotas_item_data in _quotas or []:
            quotas_item = Quotas.from_dict(quotas_item_data)

            quotas.append(quotas_item)

        quotas_response_data = cls(
            quotas=quotas,
        )

        quotas_response_data.additional_properties = d
        return quotas_response_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ...client import AuthenticatedClient, Client
from ...models.wait_instance_by_uuid_request_body import WaitInstanceByUUIDRequestBody
from ...models.wait_instance_response import WaitInstanceResponse
from ...types import Response


def _get_kwargs(
    uuid: UUID,
    *,
    body: WaitInstanceByUUIDRequestBody,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/v1/instances/{uuid}/wait",
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> WaitInstanceResponse:
    response_default = WaitInstanceResponse.from_dict(response.json())

    return response_default


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[WaitInstanceResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WaitInstanceByUUIDRequestBody,
) -> Response[WaitInstanceResponse]:
    """Wait for Instance State by UUID

     Wait for an instance to reach a certain state, by its UUID.

    If the instance is already in the desired state, the request will return
    immediately.  If the instance is not in the desired state, the request will
    block until the instance reaches the desired state or the timeout is
    reached.  If the timeout is reached, the request will fail with an error.
    If the timeout is -1, the request will block indefinitely until the
    instance reaches the desired state.

    Args:
        uuid (UUID):  Example: 123e4567-e89b-12d3-a456-426614174000.
        body (WaitInstanceByUUIDRequestBody): Wait parameters.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[WaitInstanceResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WaitInstanceByUUIDRequestBody,
) -> Optional[WaitInstanceResponse]:
    """Wait for Instance State by UUID

     Wait for an instance to reach a certain state, by its UUID.

    If the instance is already in the desired state, the request will return
    immediately.  If the instance is not in the desired state, the request will
    block until the instance reaches the desired state or the timeout is
    reached.  If the timeout is reached, the request will fail with an error.
    If the timeout is -1, the request will block indefinitely until the
    instance reaches the desired state.

    Args:
        uuid (UUID):  Example: 123e4567-e89b-12d3-a456-426614174000.
        body (WaitInstanceByUUIDRequestBody): Wait parameters.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        WaitInstanceResponse
    """

    return sync_detailed(
        uuid=uuid,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WaitInstanceByUUIDRequestBody,
) -> Response[WaitInstanceResponse]:
    """Wait for Instance State by UUID

     Wait for an instance to reach a certain state, by its UUID.

    If the instance is already in the desired state, the request will return
    immediately.  If the instance is not in the desired state, the request will
    block until the instance reaches the desired state or the timeout is
    reached.  If the timeout is reached, the request will fail with an error.
    If the timeout is -1, the request will block indefinitely until the
    instance reaches the desired state.

    Args:
        uuid (UUID):  Example: 123e4567-e89b-12d3-a456-426614174000.
        body (WaitInstanceByUUIDRequestBody): Wait parameters.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[WaitInstanceResponse]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    uuid: UUID,
    *,
    client: Union[AuthenticatedClient, Client],
    body: WaitInstanceByUUIDRequestBody,
) -> Optional[WaitInstanceResponse]:
    """Wait for Instance State by UUID

     Wait for an instance to reach a certain state, by its UUID.

    If the instance is already in the desired state, the request will return
    immediately.  If the instance is not in the desired state, the request will
    block until the instance reaches the desired state or the timeout is
    reached.  If the timeout is reached, the request will fail with an error.
    If the timeout is -1, the request will block indefinitely until the
    instance reaches the desired state.

    Args:
        uuid (UUID):  Example: 123e4567-e89b-12d3-a456-426614174000.
        body (WaitInstanceByUUIDRequestBody): Wait parameters.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        WaitInstanceResponse
    """

    return (
        await asyncio_detailed(
            uuid=uuid,
            client=client,
            body=body,
        )
    ).parsed



from typing import Literal, cast

ResponseStatus = Literal["error", "success"]

RESPONSE_STATUS_VALUES: set[ResponseStatus] = {
    "error",
    "success",
}


def check_response_status(value: str) -> ResponseStatus:
    if value in RESPONSE_STATUS_VALUES:
        return cast(ResponseStatus, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {RESPONSE_STATUS_VALUES!r}")



from typing import Literal, cast

CertificateState = Literal["error", "pending", "valid"]

CERTIFICATE_STATE_VALUES: set[CertificateState] = {
    "error",
    "pending",
    "valid",
}


def check_certificate_state(value: str) -> CertificateState:
    if value in CERTIFICATE_STATE_VALUES:
        return cast(CertificateState, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {CERTIFICATE_STATE_VALUES!r}")



from typing import Literal, cast

GetAutoscaleConfigurationsResponseStatus = Literal["error", "success", "unconfigured"]

GET_AUTOSCALE_CONFIGURATIONS_RESPONSE_STATUS_VALUES: set[GetAutoscaleConfigurationsResponseStatus] = {
    "error",
    "success",
    "unconfigured",
}


def check_get_autoscale_configurations_response_status(value: str) -> GetAutoscaleConfigurationsResponseStatus:
    if value in GET_AUTOSCALE_CONFIGURATIONS_RESPONSE_STATUS_VALUES:
        return cast(GetAutoscaleConfigurationsResponseStatus, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {GET_AUTOSCALE_CONFIGURATIONS_RESPONSE_STATUS_VALUES!r}"
    )
